*[data-v-f8fd9294]{margin:0;padding:0}nav[data-v-f8fd9294]{display:flex;justify-content:center;align-items:center;background-color:var(--secondary-background-color);font-family:var(--font);height:2.875rem;padding-left:2rem;padding-right:2rem}div[data-v-f8fd9294]{width:100%;max-width:700px}ul[data-v-f8fd9294]{display:flex;justify-content:space-between;width:100%}li[data-v-f8fd9294]{display:flex;align-items:center;list-style:none}a[data-v-f8fd9294]{text-decoration:none}img[data-v-f8fd9294]{display:flex;height:1.875rem}span[data-v-f8fd9294]{display:block;color:var(--text-color);text-align:center}.active[data-v-f8fd9294]{color:var(--text-color);font-weight:700}span[data-v-f8fd9294]:before{content:attr(data-text);display:flex;font-weight:700;height:0;overflow:hidden;visibility:hidden;-webkit-user-select:none;user-select:none;pointer-events:none}.hover-color[data-v-f8fd9294]:hover{color:var(--43711ce9)!important}.hover-bg-color[data-v-f8fd9294]:hover{background-color:var(--32628bb8)!important}.err__title[data-v-9ae37d8b],.err__msg[data-v-9ae37d8b],body{margin:0}#app{-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}
